package com.nisum.javaSparkExample;

import com.nisum.javaSparkExample.model.Customers;
import com.nisum.javaSparkExample.model.Orders;
import com.nisum.javaSparkExample.model.Users;
import org.apache.spark.sql.*;

import java.util.Arrays;
import java.util.List;

public class JavaSparkExampleApplication {

	public static void main(String[] args)
	{
		SparkSession sparkSession=SparkSession.builder()
				.master("local[*]")
				.appName("java spark example")
				//.config("spark.ui.enabled",true)
				.config("spark.sql.shuffle.partitions",2)
				.getOrCreate();

//read tha data from list
		List<Customers> customers = Arrays.asList(
				new Customers(01,"sarika","female",1000),
				new Customers(02,"sony","female",2000)

		);

		Dataset<Row> df = sparkSession.createDataFrame(customers,Customers.class);
		df.show(false);

		Dataset<Customers> customermongoData = sparkSession.read()
				.format("mongo")
				.option("uri", "mongodb://localhost:27017/mydb.customers")
				.load().as(Encoders.bean(Customers.class));
// Show data from MongoDB
		System.out.println(" Customers Data from MongoDB:");
		//customermongoData.show(false);
		customermongoData.select("customerId","name","gender","transaction_amount").show(false);



//applied transformation to dataset and read the data in console
		Dataset df1 = df.filter(functions.col("customerId")
				.isNotNull())
				.select("name","gender");
		df1.show(false);

// write the data with write() and read in console
		String outputPath = "D:/spark/customers1.csv";
		df1.write().mode(SaveMode.Overwrite)
				.format("csv")
				.option("header",true)
				.save(outputPath);

//create two collections and save into another collection

		Dataset<Customers> customersData = sparkSession.read()
				.format("mongo")
				.option("uri", "mongodb://localhost:27017/mydb.customers")
				.load().as(Encoders.bean(Customers.class));
// Show data from MongoDB
		System.out.println(" Customers Data from MongoDB:");
	   //customersData.show(false);
		customersData.select("customerId","name","gender","transaction_amount").show(false);

		Dataset<Orders> ordersmongoData = sparkSession.read()
				.format("mongo")
				.option("uri", "mongodb://localhost:27017/mydb.orders")
				.load().as(Encoders.bean(Orders.class));
// Show data from MongoDB
		System.out.println(" Orders Data from MongoDB :");
		//ordersmongoData.show(false);
		ordersmongoData.select("orderId","customerId","orderName","status").show(false);

//read the data from mongodb
		Dataset<Users> usersmongoData = sparkSession.read()
				.format("mongo")
				.option("uri", "mongodb://localhost:27017/mydb.users")
				.load().as(Encoders.bean(Users.class));
// Show data from MongoDB
		System.out.println(" users Data from MongoDB :");
		//usersmongoData.show(false);
		usersmongoData.select("userId","userName","email","address").show(false);



//		Dataset<Row> userData = customersData.join(ordersmongoData,customersData.col("customerId")
//				.equalTo(ordersmongoData.col("customerId")), "inner");
		Dataset<Row> userData=customersData.as("cust").join(ordersmongoData.as("ord"),ordersmongoData.col("customerId").equalTo(customersData.col("customerId")))
				.select(functions.col("cust.customerId"),functions.col("cust.name"),functions.col("cust.gender"),
						functions.col("cust.transaction_amount"),functions.col("ord.orderId"),functions.col("ord.orderName"),
						functions.col("ord.status"));

		userData.write()
				.format("mongo")
				.option("uri", "mongodb://localhost:27017/mydb.users")
				.mode(SaveMode.Append)
				.save();
		userData.show(false);

		sparkSession.stop();


	}

}
